#include "stdafx.h"
#include "TaskIdealize.h"
/*
Contents : 
	CVwIdealize - Options dialog 	
	VwIdealizeDlgProc - message pump for options dialog 
	OpenVwIdealize( HWND hDlg, CData * ) - external entry point to open task options dialog 
*/

BOOL CALLBACK VwIdealizeDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);

//-- Global data 
char * g_szTitleIdlHA = "Idealization - Half Amplitude";

//-----------------------------------------------------------------------------
// dialog for options edit 
class CVwIdealize : public CDialog { 
	private : 
		HBRUSH hbrButton;

		//-- Controls 
		CDlgCombo m_cmbChannel;
				
		CDlgCheck m_selDataFile;
		CDlgCheck m_selDataList;
		CDlgCheck m_selDataCurrent;
		
		CDlgCombo m_cmbModelName;
		
		CDlgCheck m_selEraseAllIdl;
		CDlgCheck m_selEraseOverlap;
		CDlgCheck m_selEraseNone;			
		
		CDlgButton m_btnColor;
		
		CDlgCheck m_selMethodHalfAmp;
		CDlgCheck m_selMethodSKM;
		CDlgCheck m_selMethodSetAmps;
		
		CDlgEdit m_edtSetValues;
		CDlgButton m_btnSetValues;

		CDlgCheck m_chkStatistics;
		CDlgCheck m_chkHistogram;

		bool m_bDataCurrent;
		bool m_bDataList;
		
	public : 		
		CStringArray * m_pacModels;

		COptionsIdealize * m_pConfig;		// ptr to structure to store control values when done

		CVwIdealize( ) {
			m_iID = IDD_TASK_IDEALIZE;
			m_pConfig = NULL;
			hbrButton = NULL;

			m_bDataCurrent = true;
			m_bDataList = true;
			}
	
		~CVwIdealize() { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			}

		//-- Initialize controls 
		void InitializeControls( ) { 
			m_cmbChannel.SetHandle( GetDlgItem(m_hDlg, IDC_SELCHANNEL), "Select the data channel to idealize");

			m_selDataFile.SetHandle( GetDlgItem(m_hDlg, IDC_DATASEL1), "Idealize entire file");
			m_selDataList.SetHandle( GetDlgItem(m_hDlg, IDC_DATASEL2), "Idealize only the current selection list");
			m_selDataCurrent.SetHandle( GetDlgItem(m_hDlg, IDC_DATASEL3), "Idealize only the current selection");
			
			m_cmbModelName.SetHandle( GetDlgItem(m_hDlg, IDC_SELMODEL), "Select the model from the list of currently open models");
			
			m_selEraseAllIdl.SetHandle( GetDlgItem(m_hDlg, IDC_IDLERASEALL), "Clear all prior idealizations in this file");
			m_selEraseOverlap.SetHandle( GetDlgItem(m_hDlg, IDC_IDLERASEOVERLAP), "Clear all prior idealizations which overlap the new idealization");
			m_selEraseNone.SetHandle( GetDlgItem(m_hDlg, IDC_IDLERASENONE), "Do not change any existing idealizations");			
			
			m_btnColor.SetHandle( GetDlgItem(m_hDlg, IDC_BTNCOLOR0), "Change the display color for this idealization");
			
			m_selMethodHalfAmp.SetHandle( GetDlgItem(m_hDlg, IDC_SEL_METHODHALFAMP), "Set all points to state with nearest amplitude value.");
			m_selMethodSKM.SetHandle( GetDlgItem(m_hDlg, IDC_SEL_METHODSKM), "Set states based on maximum likelihood calculation.");
			m_selMethodSetAmps.SetHandle( GetDlgItem(m_hDlg, IDC_SEL_METHODSETAMPS), "Set states based on half amplitude with provided levels.");
		
			m_edtSetValues.SetHandle( GetDlgItem(m_hDlg, IDC_EDT_SETVALUES), "Conductance list");
			m_btnSetValues.SetHandle( GetDlgItem(m_hDlg, IDC_BTNSETVALUES), "Open the conductance setting histogram.");

			m_chkStatistics.SetHandle( GetDlgItem(m_hDlg, IDC_CHK_STATISTICS ), "Generate transition statistics when idealization is complete" );
			m_chkHistogram.SetHandle( GetDlgItem(m_hDlg, IDC_CHKHISTOGRAM ), "Generate a results histogram when idealization is complete" );

			if( ! m_bDataCurrent  )
				EnableWindow( m_selDataCurrent.GetHandle(), false );
			if( ! m_bDataList ) 
				EnableWindow( m_selDataList.GetHandle(), false );
			}

		//-- Set all control contents and disable unused rows
		void SetControlValues( ) { 
			//-- Channel numbers
			char buf[64];
			m_cmbChannel.ClearContents();
			for( int iCh=0; iCh<m_pConfig->m_iChannels; ++iCh ) {
				sprintf( buf, "%d", iCh );
				m_cmbChannel.AddString(buf);
				}
			m_cmbChannel.SetCurSel( min(m_pConfig->m_iChannel,m_pConfig->m_iChannels-1) );
			
			//-- data - whole file, selection list, current selection 
			if( m_pConfig->m_iDataMode==2 && IsWindowEnabled( m_selDataCurrent.GetHandle() ) ) 
				m_selDataCurrent.SetCheck(true);
			else if( m_pConfig->m_iDataMode==1 && IsWindowEnabled( m_selDataList.GetHandle() ) ) 
				m_selDataList.SetCheck(true);
			else 
				m_selDataFile.SetCheck(true);
				
			//-- Fill in open model names  
			for( int i=0; i<m_pacModels->Count(); ++i ) 
				m_cmbModelName.AddString( m_pacModels->GetAt(i).Buffer() );
			m_cmbModelName.SetCurSel( 0 );	// TODO - default to option selected last time 

			//-- Erase mode - all, some, none 
			if( m_pConfig->m_iEraseMode==2 ) 
				m_selEraseNone.SetCheck(true);
			else if( m_pConfig->m_iEraseMode==1 ) 
				m_selEraseOverlap.SetCheck(true);
			else 
				m_selEraseAllIdl.SetCheck(true);

			//--color		
			m_btnColor.m_cr=m_pConfig->m_crColor;

			//-- half amp or SKM 
			if( m_pConfig->m_iMethod==IDLMTHD_SETVALUES ) 
				m_selMethodSetAmps.SetCheck(true);
			else if( m_pConfig->m_iMethod==IDLMTHD_SKM )
				m_selMethodSKM.SetCheck(true);
			else
				m_selMethodHalfAmp.SetCheck(true);
			
			m_edtSetValues.SetText(m_pConfig->m_cSetValues.ConstBuffer());
			
			//-- graphs
			m_chkStatistics.SetCheck( m_pConfig->m_bStatistics );
			m_chkHistogram.SetCheck( m_pConfig->m_bHistogram );
			}

		//-----------------------------------------------------------------------------
		bool GetControlValues(  ) { 
			//-- Get Selected idealization channel
			m_pConfig->m_iChannel = max(m_cmbChannel.GetCurSel(),0);

			//-- Get selected data 
			if( m_selDataCurrent.GetCheck() ) 
				m_pConfig->m_iDataMode=2;
			else if( m_selDataList.GetCheck() ) 
				m_pConfig->m_iDataMode=1;
			else if( m_selDataFile.GetCheck() ) 
				m_pConfig->m_iDataMode=0;
			else { 
				MessageBox( m_hDlg, "You must select a data option", g_szTitleIdlHA, MB_OK );
				return false;
				}
				
			//-- Get selected model 
			m_pConfig->m_iModelSelected=m_cmbModelName.GetCurSel();
			if( m_pConfig->m_iModelSelected<0 ) { 
				MessageBox( m_hDlg, "You must select a model", g_szTitleIdlHA, MB_OK );
				return false;
				}
			
			//-- Erase previous IDL's ? 
			if( m_selEraseNone.GetCheck() ) 
				m_pConfig->m_iEraseMode=2;
			else if( m_selEraseOverlap.GetCheck() )
				m_pConfig->m_iEraseMode=1; 
			else // default ... if( m_selEraseAllIdl.GetCheck() )
				m_pConfig->m_iEraseMode=0; 
				
			//-- SKM or 1/2 amp methods
			m_pConfig->m_iMethod = m_selMethodHalfAmp.GetCheck() ? IDLMTHD_HALFAMP 
					: (m_selMethodSKM.GetCheck() ? IDLMTHD_SKM : IDLMTHD_SETVALUES ) ;
			
			m_edtSetValues.GetText( &m_pConfig->m_cSetValues);
			
			//-- Color of IDL display
			m_pConfig->m_crColor = m_btnColor.m_cr;

			//-- graphs
			m_pConfig->m_bStatistics = m_chkStatistics.GetCheck();
			m_pConfig->m_bHistogram = m_chkHistogram.GetCheck();

			return true;
			}

		//-----------------------------------------------------------------------------
		// Events and messages 

		//-- [CANCEL]  
		void OnBtnCancel( ) { 
			EndDialog( m_hDlg, IDCANCEL );
			}
			
		//-- [SAVE] 
		void OnSave( ) { 
			if( GetControlValues() )
				EndDialog( m_hDlg, IDOK );
			}
			
		//-- [Color] selection button
		void OnBtnColorSelect( ) { 
			COLORREF acrTemp[16];
			for( int i=0; i<16; ++i ) 
				acrTemp[i]=RGB(16*i,0,0);
			
			CHOOSECOLOR cc;
			ZeroMemory(&cc, sizeof(cc));
			cc.lStructSize = sizeof(cc);
			cc.hwndOwner = m_hDlg;
			cc.lpCustColors = acrTemp;	
			cc.rgbResult = m_btnColor.m_cr;
			cc.Flags = CC_FULLOPEN | CC_RGBINIT;	// CC_FULLOPEN CC_ANYCOLOR | 
			if( ChooseColor(&cc) ) 
				m_btnColor.m_cr=cc.rgbResult;
			InvalidateRect( m_hDlg, NULL, false );
			}

		//--  [Set Values...]			
		void OnBtnSetValues() { 
			m_pConfig->m_bSetValues=true;
			OnSave();
			}

		//-- For painting the color selection buttons, create a custom brush
		HBRUSH CreateButtonBrush( ) { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			return hbrButton = CreateSolidBrush( m_btnColor.m_cr );
			}

		void DisableDataCurrent() { 
			m_bDataCurrent=false;
			}

		void DisableDataList() { 
			m_bDataList = false;
			}
	};

//-----------------------------------------------------------------------------
CVwIdealize * g_pVwIdealize;		// Modal - one and only instance

//-----------------------------------------------------------------------------
void OpenVwIdealize( HWND hDlgVwData, CData * pData, bool bList, const char * pchList, 
							bool bSelect, int iSeg0, int iOffset0, int iSeg1, int iOffset1 ) {  
	
	//-- Attempt to lock data 
	if( ! pData->SetStatus(1) ) {
		MessageBox( hDlgVwData, "The data is currently in use by another task.", g_szTitleIdlHA, MB_OK );
		return;
		}
		
	//-- create structure to pass option to / from dialogue.
	COptionsIdealize * pConfig = new COptionsIdealize;		
	pConfig->m_iChannel=0;
	pConfig->m_iChannels=pData->m_oConfig.ChannelCount();
	pConfig->m_iDataMode=0;
	pConfig->m_iEraseMode=0;
	pConfig->m_crColor=RGB(255,64,64);
	pConfig->m_iMethod=1;
	pConfig->m_bSetValues=false;

	pConfig->RegistryLoad();
	
	CPtrArray ahVwModels;
	CStringArray acModels;

	//-- check if any open models
	int iWin = WindowListFindID( IDD_MODELVIEW, 0 );
	if( iWin == -1 ) {
		MessageBox( hDlgVwData, "There are no open models.\nA model is required for idealization.\n", g_szTitleIdlHA, MB_OK);
		pData->SetStatus(0);
		return;
		}

	//-- iterate all open models - get model file name and window handle 
	while( iWin != -1 ) { 
		HWND hwndModel = WindowListHandle( iWin );
		char * pchFile = (char *)(((CDialog *) GetWindowLong( GetParent(hwndModel), 0 ))->GetData( 1, NULL ));
		acModels.AddString( pchFile );
		ahVwModels.AddPtr( hwndModel );
		iWin = WindowListFindID( IDD_MODELVIEW, iWin+1 );
		}

	//-- create dialogue to get task options 	
	g_pVwIdealize = new CVwIdealize;
	g_pVwIdealize->m_pConfig = pConfig;
	
	g_pVwIdealize->m_pacModels = &acModels;
	
	//-- Remove the current selection option if no data 
	if( ! bSelect || (iSeg0==iSeg1 && iOffset0>=iOffset1) || (iSeg0>iSeg1) ) 
		g_pVwIdealize->DisableDataCurrent();	
		
	if( ! bList || pchList==NULL || strlen(pchList)==0 ) 
		g_pVwIdealize->DisableDataList();		
	
	//-- present dialogue, execute if [OK]
	if( DialogBox( GetInstance(), (LPCTSTR)IDD_TASK_IDEALIZE, hDlgVwData, (DLGPROC)VwIdealizeDlgProc ) == IDOK ) { 
		pConfig->RegistrySave();

		//-- If clicked on [...] to end dialog and open histogram ... 
		if( pConfig->m_bSetValues ) { 
			//-- create window
			HWND hwndHist = OpenHistogramView( "Idealization amplitude setting" );
			
			CSelectionSet oSel;
			oSel.Setup( hDlgVwData, pConfig->m_iDataMode, pData, pConfig->m_iChannel );
			
			//-- find min and max in all ranges
			double dMin, dMax;
			oSel.CalculateMinMax( pConfig->m_iChannel, &dMin, &dMax );
			
			//-- Create empty histogram
			CHistogram * pHistogram = new CHistogram( 400, dMin, (dMax-dMin)/400.0 );
			pHistogram->SetTitles( "Data Histogram", "Conductance", "Event Count" );
			
			//-- add data to histogram bins
			//TODO - GetRange makes oSel copy of data, perhaps GetRangePtr and interleave
			int iRanges=oSel.m_aRanges.Count();
			for( int iRange=0; iRange<iRanges; ++iRange ) { 
				oSel.GetRange( iRange );
				pHistogram->AddData( oSel.CurrentSize(), oSel.m_adData );
				}
				
			//-- transfer histogram object to view 
			PostMessage( hwndHist, WM_IDLHIST_OPEN, 0, (LPARAM)pHistogram );
		    
			delete pConfig;
			pData->SetStatus(0);
			return;
			}
		
		//----- clicked on [OK] to start idealization
		//-- Open the task & progress view 	
		CVwTaskIdealize * pVwTaskIdl = new CVwTaskIdealize( );
		pVwTaskIdl->m_pOptionsIdealize = pConfig;			// transfer ownership to task (for later delete)
		pVwTaskIdl->m_hDlgVwData = hDlgVwData;
		
		//-- reference the selected model to create array of classes for conductances and stddev
		//TODO - lock the model, then have task access model info first and unlock model.
		pVwTaskIdl->m_pModel = (CModel *)( ((CDialog *) GetWindowLong(GetParent((HWND)ahVwModels[pConfig->m_iModelSelected]),0) )->GetData( 2, NULL ));
		
		//-- dataset setup
		if( pVwTaskIdl->m_oSelSet.Setup( hDlgVwData, pConfig->m_iDataMode, pData, pConfig->m_iChannel ) ) { 

			//-- Erase Mode m_iEraseMode;	(0:all) (1:overlapping) (2:none)
			int iSegs = pData->m_aSegments.Count();
			if( pConfig->m_iEraseMode==0 || (pConfig->m_iEraseMode==1 && pConfig->m_iDataMode==0)) {				
				for( int iSeg=0; iSeg<iSegs; ++iSeg )		// Erase All or Erase overlapping + IdealizeAll
					pData->Segment(iSeg)->m_aIdl.DeleteAll();
				}
			else if( pConfig->m_iEraseMode==1 ) {			// Erase Overlapping
				for( int iSel=0; iSel<pVwTaskIdl->m_oSelSet.m_aRanges.Count(); ++iSel ) {
					CSelectionSetRange * pRange = pVwTaskIdl->m_oSelSet.Range(iSel);
					pData->Segment(pRange->m_iSeg)->DeleteOverlappingIdls( pRange->m_iMin, pRange->m_iMax );
					}
				}
			
			//-- Create separate thread to do idealization
			HWND hVwTask = pVwTaskIdl->CreateTaskWindow( GetClientWin(), GetChildClassName() );
			pVwTaskIdl->m_hThread = CreateThread( NULL, 0, TaskIdealize, pVwTaskIdl, 0, NULL );
			}
		else 
			MessageBox( hDlgVwData, "Error setting data source", g_szTitleIdlHA, MB_OK);
		} 
	else 
		delete pConfig;
	pData->SetStatus(0);
	}

//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK VwIdealizeDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	int wmId, wmEvent;
	
	switch( iMsg ) { 
		//----- DIALOG INIT -  Control Initialization
		case WM_INITDIALOG :		
			g_pVwIdealize->m_hDlg=hDlg;			
			
			//-- Clear Existing  Values
			g_pVwIdealize->InitializeControls();
			g_pVwIdealize->SetControlValues();

			return TRUE;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 

			if( wmId == IDC_BTNCOLOR0 ) 
				g_pVwIdealize->OnBtnColorSelect( );
				
			switch( wmId ) { 
				case IDOK :					//-- [OK]
					g_pVwIdealize->OnSave( );
					break;

				case IDCANCEL :				//-- [CANCEL]
					g_pVwIdealize->OnBtnCancel( );
					break;
					
				case IDC_BTNSETVALUES : 
					g_pVwIdealize->OnBtnSetValues();
				}
				
			return TRUE;

		case WM_CTLCOLORBTN :
			wmId = GetDlgCtrlID( (HWND)lParam );
			if( wmId == IDC_BTNCOLOR0  )
				return (BOOL) g_pVwIdealize->CreateButtonBrush();
			return FALSE;

		case WM_PAINT : 
			/*
			{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hDlg, &ps);
							
			//----- Manual Drawing 
				
			//----- Done
			EndPaint(hDlg, &ps);
			}
			*/
			return false;

		case WM_DESTROY : 
			delete g_pVwIdealize;
			WindowListRemove( hDlg );
			return TRUE;
			
		default : 
			break;
		}
	return FALSE;
	} 


